<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Model;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Exception\CouldNotDeleteException;
use Extensa\Speedy\Api\Data\AddressSearchResultsInterfaceFactory;
use Extensa\Speedy\Api\Data\AddressInterface;
use Extensa\Speedy\Api\Data\AddressInterfaceFactory;
use Extensa\Speedy\Api\AddressRepositoryInterface;
use Extensa\Speedy\Model\ResourceModel\Address as AddressResource;
use Extensa\Speedy\Model\ResourceModel\Address\CollectionFactory;

class AddressRepository implements AddressRepositoryInterface
{
    /**
     * @var AddressResource
     */
    private $addressResource;

    /**
     * @var AddressFactory
     */
    private $addressFactory;

    /**
     * @var CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var AddressSearchResultsInterfaceFactory
     */
    private $searchResultsFactory;

    /**
     * @var DataObjectHelper
     */
    protected $dataObjectHelper;

    /**
     * @var AddressInterfaceFactory
     */
    protected $addressInterfaceFactory;

    /**
     * AddressRepository constructor.
     * @param AddressResource $addressResource
     * @param AddressFactory $addressFactory
     * @param CollectionFactory $collectionFactory
     * @param AddressSearchResultsInterfaceFactory $searchResultsFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param AddressInterfaceFactory $addressInterfaceFactory
     */
    public function __construct(
        AddressResource $addressResource,
        AddressFactory $addressFactory,
        CollectionFactory $collectionFactory,
        AddressSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        AddressInterfaceFactory $addressInterfaceFactory
    ) {
        $this->addressResource = $addressResource;
        $this->addressFactory = $addressFactory;
        $this->collectionFactory = $collectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->addressInterfaceFactory = $addressInterfaceFactory;
    }

    /**
     * Save address data
     *
     * @param AddressInterface $address
     * @return int
     * @throws CouldNotSaveException
     */
    public function save(AddressInterface $address)
    {
        try {
            $this->addressResource->save($address);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the Speedy address: %1',
                $exception->getMessage()
            ));
        }

        return $address->getId();
    }

    /**
     * Get Speedy address by ID
     *
     * @param int $addressId
     * @return Address
     * @throws NoSuchEntityException
     */
    public function getById($addressId)
    {
        $address = $this->addressFactory->create();

        $this->addressResource->load($address, $addressId, 'address_id');

        if (!$address->getId()) {
            throw new NoSuchEntityException(__('Speedy address with id "%1" does not exist.', $addressId));
        }

        return $address;
    }

    /**
     * Load Speedy address data collection by given search criteria
     *
     * @param \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
     * @return \Magento\Customer\Api\Data\AddressSearchResultsInterface
     */
    public function getList(\Magento\Framework\Api\SearchCriteriaInterface $searchCriteria)
    {
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($searchCriteria);

        $collection = $this->collectionFactory->create();
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            foreach ($filterGroup->getFilters() as $filter) {
                if ($filter->getField() === 'store_id') {
                    $collection->addStoreFilter($filter->getValue(), false);
                    continue;
                }
                $condition = $filter->getConditionType() ?: 'eq';
                $collection->addFieldToFilter($filter->getField(), [$condition => $filter->getValue()]);
            }
        }
        $searchResults->setTotalCount($collection->getSize());
        $sortOrders = $searchCriteria->getSortOrders();
        if ($sortOrders) {
            /** @var SortOrder $sortOrder */
            foreach ($sortOrders as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());
        $addresses = [];
        /** @var Address $addressModel */
        foreach ($collection as $addressModel) {
            $adddressData = $this->addressInterfaceFactory->create();
            $this->dataObjectHelper->populateWithArray(
                $adddressData,
                $addressModel->getData(),
                'Extensa\Speedy\Api\Data\AddressInterface'
            );
            $addresses[] = $addressModel;
        }
        $searchResults->setItems($addresses);
        return $searchResults;
    }

    /**
     * Delete address
     *
     * @param AddressInterface $address
     * @return bool
     * @throws CouldNotDeleteException
     */
    public function delete(AddressInterface $address)
    {
        try {
            $this->addressResource->delete($address);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the Speedy address: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * Delete Speedy address by given ID
     *
     * @param int $addressId
     * @return bool
     */
    public function deleteById($addressId)
    {
        return $this->delete($this->getById($addressId));
    }
}
