<?php
/**
 * @author Extensa Web Development Ltd. <support@extensadev.com>
 */

namespace Extensa\Speedy\Model;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Extensa\Speedy\Api\Data\TablerateSearchResultsInterfaceFactory;
use Extensa\Speedy\Api\Data\TablerateInterface;
use Extensa\Speedy\Api\Data\TablerateInterfaceFactory;
use Extensa\Speedy\Api\TablerateRepositoryInterface;
use Extensa\Speedy\Model\ResourceModel\Tablerate as TablerateResource;
use Extensa\Speedy\Model\ResourceModel\Tablerate\CollectionFactory;

class TablerateRepository implements TablerateRepositoryInterface
{
    /**
     * @var TablerateResource
     */
    private $tablerateResource;

    /**
     * @var TablerateFactory
     */
    private $tablerateFactory;

    /**
     * @var CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var TablerateSearchResultsInterfaceFactory
     */
    private $searchResultsFactory;

    /**
     * @var DataObjectHelper
     */
    protected $dataObjectHelper;

    /**
     * @var TablerateInterfaceFactory
     */
    protected $tablerateInterfaceFactory;

    /**
     * TablerateRepository constructor.
     * @param TablerateResource $tablerateResource
     * @param TablerateFactory $tablerateFactory
     * @param CollectionFactory $collectionFactory
     * @param AddressSearchResultsInterfaceFactory $searchResultsFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param TablerateInterfaceFactory $tablerateInterfaceFactory
     */
    public function __construct(
        TablerateResource $tablerateResource,
        TablerateFactory $tablerateFactory,
        CollectionFactory $collectionFactory,
        TablerateSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        TablerateInterfaceFactory $tablerateInterfaceFactory
    ) {
        $this->tablerateResource = $tablerateResource;
        $this->tablerateFactory = $tablerateFactory;
        $this->collectionFactory = $collectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->tablerateInterfaceFactory = $tablerateInterfaceFactory;
    }

    /**
     * Save tableRate data
     *
     * @param TablerateInterface $tableRate
     * @return int
     * @throws CouldNotSaveException
     */
    public function save(TablerateInterface $tableRate)
    {
        try {
            $this->tablerateResource->save($tableRate);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the Speedy table rate: %1',
                $exception->getMessage()
            ));
        }

        return $tableRate->getId();
    }

    /**
     * Get Speedy Table Rate by ID
     *
     * @param int $serviceId
     * @return tableRate
     * @throws NoSuchEntityException
     */
    public function getById($tableRate)
    {
        $tableRateFactory = $this->tablerateFactory->create();

        $this->tablerateResource->load($tableRateFactory, $tableRate, 'table_rate_id');

        if (!$tableRateFactory->getId()) {
            throw new NoSuchEntityException(__('Speedy table rate with id "%1" does not exist.', $tableRate));
        }

        return $tableRateFactory;
    }

    /**
     * Load Speedy tableRate data collection by given search criteria
     *
     * @param \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
     * @return \Extensa\Speedy\Api\Data\AddressSearchResultsInterface
     */
    public function getList(\Magento\Framework\Api\SearchCriteriaInterface $searchCriteria)
    {
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($searchCriteria);

        $collection = $this->collectionFactory->create();
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            foreach ($filterGroup->getFilters() as $filter) {
                if ($filter->getField() === 'store_id') {
                    $collection->addStoreFilter($filter->getValue(), false);
                    continue;
                }
                $condition = $filter->getConditionType() ?: 'eq';
                $collection->addFieldToFilter($filter->getField(), [$condition => $filter->getValue()]);
            }
        }
        $searchResults->setTotalCount($collection->getSize());
        $sortOrders = $searchCriteria->getSortOrders();
        if ($sortOrders) {
            /** @var SortOrder $sortOrder */
            foreach ($sortOrders as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());

        /** @var Tablerate $tablerateModel */
        $tableRate = [];
        foreach ($collection as $tableRateModel) {
            $tableRateData = $this->tablerateInterfaceFactory->create();

            $this->dataObjectHelper->populateWithArray(
                $tableRateData,
                $tableRateModel->getData(),
                'Extensa\Speedy\Api\Data\TablerateInterface'
            );

            $tableRate[] = $tableRateModel;
        }

        $searchResults->setItems($tableRate);
        return $searchResults;
    }

    public function getFirstResult(\Magento\Framework\Api\SearchCriteriaInterface $searchCriteria)
    {
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($searchCriteria);

        $collection = $this->collectionFactory->create();
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            foreach ($filterGroup->getFilters() as $filter) {
                $condition = $filter->getConditionType() ?: 'eq';
                $collection->addFieldToFilter($filter->getField(), [$condition => $filter->getValue()]);
            }
        }

        return $collection->getFirstItem();
    }
}
